/* FILE: atlas.h                 -*-Mode: c++-*-
 *
 * Abstract atlas class, derived from OXS extension class.
 *
 */

#ifndef _OXS_ATLAS
#define _OXS_ATLAS

#include <string>

#include "ext.h"
#include "threevector.h"
#include "util.h"
/* End includes */

class Oxs_Atlas:public Oxs_Ext {
protected:
  Oxs_Atlas(const char* name,      // Child instance id
	    Oxs_Director* newdtr); // App director
  Oxs_Atlas(const char* name,
	    Oxs_Director* newdtr,
	    Tcl_Interp* safe_interp, // Safe interpreter
	    const char* argstr);   // MIF block argument string
public:
  virtual ~Oxs_Atlas() {}

  virtual void GetWorldExtents(Oxs_Box &mybox) const =0;
  /// Fills mybox with bounding box for the atlas

  virtual INT4m GetRegionId(const ThreeVector& point) const =0;
  /// Returns the id number for the first region in
  /// "region_list" containing point.  The return
  /// value is -1 if the point is not contained in
  /// the atlas.

  virtual INT4m GetRegionId(const string& name) const =0;
  /// Given a region id string (name), returns
  /// the corresponding region id index.  If
  /// "name" is not included in the atlas, then
  /// -1 is returned.

  virtual INT4m GetRegionName(UINT4m id,string& name) const =0;
  /// Given an id number, fills in "name" with
  /// the corresponding region id string.  Returns
  /// 0 on success, 1 if id is invalid.

  virtual UINT4m GetRegionCount() const =0;
  /// Valid RegionId numbers range from 0 to GetRegionCount -1,
  /// inclusive.

};

#endif // _OXS_ATLAS
