/* FILE: cubicanisotropy.h            -*-Mode: c++-*-
 *
 * Cubic Anisotropy, derived from Oxs_Energy class.
 */

#ifndef _OXS_CUBICANISOTROPY
#define _OXS_CUBICANISOTROPY

#include <memory>   // auto_ptr template

#include "nb.h"
#include "threevector.h"
#include "energy.h"
#include "key.h"
#include "simstate.h"
#include "mesh.h"
#include "meshvalue.h"
#include "scalarfieldinit.h"
#include "vectorfieldinit.h"

/* End includes */

class Oxs_CubicAnisotropy:public Oxs_Energy {
private:
  auto_ptr<Oxs_ScalarFieldInit> K1_init;
  auto_ptr<Oxs_VectorFieldInit> axis1_init;
  auto_ptr<Oxs_VectorFieldInit> axis2_init;
  mutable UINT4m mesh_id;
  mutable Oxs_MeshValue<REAL8m> K1;
  mutable Oxs_MeshValue<ThreeVector> axis1;
  mutable Oxs_MeshValue<ThreeVector> axis2;
  /// K1, axis1 and axis2 are cached values filled by corresponding
  /// *_init members when a change in mesh is detected.
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_CubicAnisotropy(const char* name,  // Child instance id
		      Oxs_Director* newdtr, // App director
		      Tcl_Interp* safe_interp, // Safe interpreter
		      const char* argstr);  // MIF input block parameters

  virtual ~Oxs_CubicAnisotropy() {}

  virtual void GetEnergyAndField(const Oxs_SimState& state,
                                 Oxs_MeshValue<REAL8m>& energy,
                                 Oxs_MeshValue<ThreeVector>& field
                                 ) const;
};


#endif // _OXS_CUBICANISOTROPY
